@@ -2,6 +2,8 @@
 # Use of this source code is governed by a BSD-style
 # license that can be found in the LICENSE file.
 
+package Spp::OptSppAst;
+
 use Exporter;
 our @ISA       = qw(Exporter);
 our @EXPORT_OK = qw(opt_spp_ast);
@@ -17,14 +19,16 @@ sub opt_spp_ast {
    my $atoms = [];
 
    # when single spec, data structure is same with multi
-   if (is_spec($ast)) {
-      my $spec = opt_spp_spec($ast->[1]);
-      return [ $spec ];
+   if (is_atom($ast)) {
+      my ($name, $spec) = @{$ast};
+      if ($name eq 'Spec') {
+         return [ opt_spp_spec($spec) ];
+      }
    }
    for my $atom (@{$ast}) {
-      my ($name, $rule) = @{$atom};
-      if ($name ~~ 'Spec') {
-         push @{$atoms}, opt_spp_spec($rule);
+      my ($name, $spec) = @{$atom};
+      if ($name eq 'Spec') {
+         push @{$atoms}, opt_spp_spec($spec);
       }
    }
    return $atoms;
@@ -32,11 +36,12 @@ sub opt_spp_ast {
 
 sub opt_spp_spec {
    my $atoms = shift;
-   my ($sym, @rules) = @{$atoms};
-   error("spec name not sym") if !is_sym($sym);
-   my $name = $sym->[1];
-   my $opt_rules = opt_spp_rules([@rules]);
-   return [ $name, $opt_rules ];
+   my $token = $atoms->[0];
+   die("spec name not token") if !is_token($token);
+   my $name = $token->[1];
+   my $rules = rest($atoms);
+   my $opt_rules = opt_spp_rules($rules);
+   return [$name, $opt_rules];
 }
 
 sub opt_spp_rules {
@@ -45,7 +50,7 @@ sub opt_spp_rules {
    if (len($opt_rules) == 1) {
       return $opt_rules->[0];
    }
-   return [ 'Rules', $opt_rules ];
+   return ['Rules', $opt_rules];
 }
 
 sub opt_spp_atoms {
@@ -69,66 +74,68 @@ sub opt_spp_atom {
       when ('Str')     { return opt_spp_str($value) }
       when ('String')  { return opt_spp_str($value) }
       when ('Keyword') { return opt_spp_keyword($value) }
-      when ('Point')   { return ['Char', chr(hex($value))] }
+      when ('Point')   { return opt_spp_point($value) }
       when ('Chclass') { return opt_spp_chclass($value) }
       when ('Look')    { return opt_spp_look($value) }
       when ('Token')   { return opt_spp_token($value) }
       when ('Expr')    { return opt_spp_expr($value) }
-      when ('Assert')  { return $atom }
-      when ('Any')     { return $atom }
-      when ('Till')    { return $atom }
-      when ('Not')     { return $atom }
       when ('Sym')     { return opt_spp_sym($value) }
-      when ('Int')     { return ['Int', 0 + $value ] }
-      when ('Expr')    { return opt_spp_expr($value) }
       when ('Array')   { return opt_spp_array($value) }
-      default          { error("unknown Rule: $name to opt") }
+      when (['Assert','Any','Till','Not','Int']) {
+         return $atom
+      }
+      default { die "unknown Spp atom to opt: |$name|" }
    }
 }
 
+sub opt_spp_point {
+   my $point = shift;
+   return ['Char', chr(hex($point))];
+}
+
 sub opt_spp_group {
    my $atoms  = shift;
-   my $_atoms = opt_spp_atoms($atoms);
-   if (len($_atoms) == 1) {
-      return $_atoms->[0];
+   my $opt_atoms = opt_spp_atoms($atoms);
+   if (len($opt_atoms) == 1) {
+      return $opt_atoms->[0];
    }
-   return [ 'Group', $_atoms ];
+   return ['Group', $opt_atoms];
 }
 
 sub opt_spp_branch {
    my $atoms  = shift;
-   my $_atoms = opt_spp_atoms($atoms);
-   return $_atoms->[0] if len($_atoms) == 1;
-   return [ 'Branch', $_atoms ];
+   my $opt_atoms = opt_spp_atoms($atoms);
+   return $opt_atoms->[0] if len($opt_atoms) == 1;
+   return ['Branch', $opt_atoms];
 }
 
 sub opt_spp_lbranch {
    my $atoms  = shift;
    my $opt_atoms = opt_spp_atoms($atoms);
    return $opt_atoms->[0] if len($opt_atoms) == 1;
-   return [ 'Lbranch', $opt_atoms ];
+   return ['Lbranch', $opt_atoms];
 }
 
 sub opt_spp_keyword {
   my $keyword = shift;
-  my $str = substr($keyword, 1);
+  my $str = rest($keyword);
   return ['Char', $str] if len($str) == 1;
   return ['Str',  $str];
 }
 
 sub opt_spp_cclass {
    my $cclass = shift;
-   return [ 'Cclass', substr($cclass, 1) ];
+   return ['Cclass', tail($cclass)];
 }
 
 sub opt_spp_char {
   my $char = shift;
-  return ['Char', opt_spp_escape_char($char) ];
+  return ['Char', opt_spp_escape_char($char)];
 }
 
 sub opt_spp_escape_char {
   my $str = shift;
-  my $char = substr($str, 1);  
+  my $char = tail($str);  
   given ($char) {
     when ('b') { return ' '  }
     when ('f') { return "\f" }
@@ -161,21 +168,21 @@ sub opt_spp_catom {
       when ('Cclass') { return opt_spp_cclass($value) }
       when ('Range')  { return opt_spp_range($value) }
       when ('Char')   { return opt_spp_char($value) }
-      when ('Cchar')  { return [ 'Char', $value ] }
-      default         { error("unknown cclass atom $name") }
+      when ('Cchar')  { return ['Char', $value] }
+      default         { die("unknown cclass atom $name") }
    }
 }
 
 sub opt_spp_range {
    my $range = shift;
-   return [ 'Range', [ split('-', $range) ] ];
+   return ['Range', [ split('-', $range) ]];
 }
 
 sub opt_spp_look {
    my $atoms = shift;
    my $rept  = $atoms->[0][1];
-   return [ '@Rept', $rept ] if (len($atoms) == 1);
-   return [ '@Look', $rept ];
+   return ['_rept', $rept] if len($atoms) == 1;
+   return ['_look', $rept];
 }
 
 sub gather_spp_tillnot {
@@ -196,16 +203,16 @@ sub gather_spp_tillnot {
       else {
          if (!is_tillnot($atom)) {
             my $name = $cache->[0];
-            $cache = [ $name, $atom ];
+            $cache = [$name, $atom];
             push @opt_atoms, $cache;
             $flag = 0;
          }
          else {
-            error('Till/Not duplicate');
+            die('Till/Not duplicate');
          }
       }
    }
-   if ($flag > 0) { error("Till/Not without token!") }
+   if ($flag > 0) { die("Till/Not without token!") }
    return [@opt_atoms];
 }
 
@@ -218,7 +225,7 @@ sub gather_spp_look {
    for my $atom (@{$atoms}) {
       if ($flag == 0) {
          if (is_look($atom)) {
-            error("Look token less prefix atom: $atom");
+            die("Look token less prefix atom: $atom");
          }
          else {
             $cache = $atom;
@@ -237,16 +244,16 @@ sub gather_spp_look {
       }
       else {
          if (!is_look($atom)) {
-            $cache = [ 'Look', [ $look, $cache, $atom ] ];
+            $cache = ['Look', [$look, $cache, $atom]];
             push @opt_atoms, $cache;
             $flag = 0;
          }
          else {
-            error("Look token repeat");
+            die("Look token repeat");
          }
       }
    }
-   if ($flag > 1) { error('Look without atom!') }
+   if ($flag > 1) { die('Look without atom!') }
    push @opt_atoms, $cache if $flag == 1;
    return [@opt_atoms];
 }
@@ -259,7 +266,7 @@ sub gather_spp_rept {
    for my $atom (@{$atoms}) {
       if ($flag == 0) {
          if (is_rept($atom)) {
-            error("rept without token");
+            die("rept without token");
          }
          else {
             $cache = $atom;
@@ -269,7 +276,7 @@ sub gather_spp_rept {
       else {
          if (is_rept($atom)) {
             my $rept = $atom->[1];
-            push @opt_atoms, [ 'Rept', [ $rept, $cache ] ];
+            push @opt_atoms, ['Rept', [$rept, $cache]];
             $flag = 0;
          }
          else {
@@ -284,11 +291,11 @@ sub gather_spp_rept {
 
 sub opt_spp_token {
    my $name = shift;
-   my $char = substr($name, 0, 1);
-   return [ 'Ntoken', $name ] if is_upper($char);
-   return [ 'Ctoken', $name ] if is_lower($char);
-   return [ 'Rtoken', $name ] if $char eq '_';
-   error("Unknown token: <$name> to Opt");
+   my $char = first($name);
+   return ['Ntoken', $name] if is_upper($char);
+   return ['Ctoken', $name] if is_lower($char);
+   return ['Rtoken', $name] if $char eq '_';
+   die("Unknown token: <$name> to Opt");
 }
 
 sub opt_spp_expr {
@@ -299,21 +306,21 @@ sub opt_spp_expr {
       $opt_atoms->[0] = $action->[1];
       return [ 'Expr', $opt_atoms ];
    }
-   error("Expr not action: {$action->[1]}");
+   die("Expr not action: {$action->[1]}");
 }
 
 sub opt_spp_sym {
    my $name = shift;
    return ['false'] if $name eq 'false';
    return ['true']  if $name eq 'true';
-   return [ 'Sym', $name ];
+   return ['Sym', $name];
 }
 
 sub opt_spp_array {
    my $atoms = shift;
-   return [ 'Array', [] ] if is_chars($atoms);;
+   return ['Array', []] if is_perl_str($atoms);;
    my @opt_atoms = map { opt_spp_atom($_) } @{$atoms};
-   return [ 'Array', [ @opt_atoms ]];
+   return ['Array', [@opt_atoms]];
 }
 
 sub opt_spp_str {
@@ -328,13 +335,12 @@ sub opt_spp_str {
          my $char = opt_spp_escape_char($value);
          push @{$opt_atoms}, $char;
        }
-       default { error("unknown string atom: <$name>") }
+       default { die("unknown string atom: <$name>") }
      }
    }
    my $str = join('', @{$opt_atoms});
-   return [ 'Char', $str ] if len($str) == 1;
-   return [ 'Str', join('', @{$opt_atoms}) ];
+   return ['Char', $str ] if len($str) == 1;
+   return ['Str', join('', @{$opt_atoms})];
 }
 
-
 1;
